#############################################################################
# Author:                                                                   #
# ------                                                                    #
#  Anton Kokalj                                  Email: Tone.Kokalj@ijs.si  #
#  Department of Physical and Organic Chemistry  Phone: x 386 1 477 3523    #
#  Jozef Stefan Institute                          Fax: x 386 1 477 3811    #
#  Jamova 39, SI-1000 Ljubljana                                             #
#  SLOVENIA                                                                 #
#                                                                           #
# Source: $XCRYSDEN_TOPDIR/Tcl/gifAnim.tcl
# ------                                                                    #
# Copyright (c) 1996-2003 by Anton Kokalj                                   #
#############################################################################

proc gifAnimWidPack {frame can} {
    global gifAnim

    if { $gifAnim(button_text) == "Animated GIF/MPEG >>" } {
	pack $frame -side top -padx 0 -pady 2
	set gifAnim(button_text) "<< Animated GIF/MPEG"
    } else {
	pack forget $frame
	set gifAnim(button_text) "Animated GIF/MPEG >>"
    }
}


proc gifAnimMake {can {play_frame ""} {record_button ""} {outfile {}}} {
    global gifAnim xcMisc mesa_bg system
    
    if { $gifAnim(make_gifAnim) == 0 } {
	
	# --------------------------------------------------
	# The "Start Recording Animated-GIF/MPEG" button was pressed
	#
	# This means we start recoring. Also change the label of the 
	# button to "Stop Recording and Save"
	# --------------------------------------------------

	set gifAnim(make_gifAnim) 1
	set gifAnim(filelist)     {}  
	set gifAnim(nframe)       0
	
	if { $play_frame != "" } {
	    set gifAnim(make_text)        "Stop Recording and Save"
	    set gifAnim(play_frame_bg)    [$play_frame cget -bg]
	    set gifAnim(record_button_bg) [$record_button cget -bg]	
	    $play_frame    configure -background \#ff4444
	    $record_button configure -background \#ff4444
	    $record_button configure -relief sunken
	}
    } else {

	# --------------------------------------------------
	# The "Stop Recording and Save" button was pressed
	#
	# This means we save the Animated-GIF/MPEG file. Also change the 
	# label of the  button to "Start Recording Animated-GIF/MPEG"
	# --------------------------------------------------

	set gifAnim(make_gifAnim) 0
	
	if { $play_frame != "" } {
	    set gifAnim(make_text)   "Start Recording Animated-GIF/MPEG"
	    $play_frame    configure -background $gifAnim(play_frame_bg)
	    $record_button configure -background $gifAnim(record_button_bg)
	    $record_button configure -relief raised
	}

	if { $gifAnim(movie_format) == "mpeg" } {
	    set separator \n
	} else {
	    set separator " "
	}
	set first_frame [lindex $gifAnim(filelist) 0]
	set last_frame  [lindex $gifAnim(filelist) end]
	set first {}
	set last  {}
	if { $gifAnim(ntime_first_frame) > 1 } {
	    for {set i 1} {$i < $gifAnim(ntime_first_frame)} {incr i} {
		append first "${first_frame}$separator"
	    }
	}
	if { $gifAnim(ntime_last_frame) > 1 } {
	    for {set i 1} {$i < $gifAnim(ntime_last_frame)} {incr i} {
		append last "${last_frame}$separator"
	    }
	}
	set filelist [format "%s${separator}%s${separator}%s" $first $gifAnim(filelist) $last]
	

	if { $gifAnim(movie_format) == "gif" } {

	    # --------------------------------------------------
	    # create Animated GIF
	    # --------------------------------------------------
	    if { $outfile == "" } {
		set outfile [tk_getSaveFile -defaultextension .gif \
				 -filetypes { 
				     {{GIF File}  {.gif}}
				     {{All Files} {.*}}
				 } \
				 -initialdir  $system(PWD) \
				 -title       "Save Animated GIF File"]	    
		if { $outfile == "" } {
		    return
		}
	    }
	    
	    set flags ""
	    if { $gifAnim(gif_minimize) } {
		append flags " -minimize"
	    }
	    if { $gifAnim(gif_global_colormap) } {
		append flags " -globalmap"
	    }
	    if { $gifAnim(gif_transp) } {
		if { ! [info exists mesa_bg(current)] } {
		    set mesa_bg(current) \#000000
		}
		if { [string index $mesa_bg(current) 0] == "#" } {
		    set bg_color $mesa_bg(current)
		} else {
		    set bg_color [rgb_f2h $mesa_bg(current)]
		}
		# the "-trans" flag seems not to work in WHIRLGIF
		append flags " -trans $bg_color"
	    }
	    set comlin [format "%s %s %s %s %s" $xcMisc(whirlgif) $flags -o $outfile $filelist]
	    
	    if { $gifAnim(temp_files_dir) == "pwd" } {
		cd $system(PWD)
	    } else {
		cd $system(SCRDIR)
	    }
	    if { $gifAnim(edit_param) } {
		gifAnim:editParam $comlin
	    }
	    set cw [DisplayUpdateWidget "Encoding" "Encoding the Animated-GIF movie"]
	    eval xcCatchExecRedirectStdErr $comlin
	    destroy $cw
	} else {

	    # --------------------------------------------------
	    # create MPEG
	    # --------------------------------------------------

	    if { $outfile == "" } {
		set outfile [tk_getSaveFile -defaultextension .mpg \
				 -filetypes { 
				     {{MPEG File}  {.mpg}}
				     {{All Files}  {.*}}
				 } \
				 -initialdir  $system(PWD) \
				 -title       "Save MPEG File"]
		if { $outfile == "" } {
		    return
		}
	    }
	    
	    if { $gifAnim(temp_files_dir) == "pwd" } {
		cd $system(PWD)
	    } else {
		cd $system(SCRDIR)
	    }	    
	    
	    set pfile [mpegCreateParamFile $outfile $system(SCRDIR) $filelist]
	    set cw [DisplayUpdateWidget "Encoding" "Encoding the MPEG movie"]
	    xcCatchExecRedirectStdErr $xcMisc(mpeg_encode) $pfile
	    destroy $cw
	}
    }
}

	
proc gifAnimPrintCurrent {can} {
    global gifAnim xcMisc mesa_bg system

    if { $gifAnim(temp_files_dir) == "pwd" } {
	set dir $system(PWD)
    } else {
	set dir $system(SCRDIR)
    }

    set head anim-$gifAnim(nframe)

    if { $gifAnim(movie_format) == "gif" } {
	set file $head.gif
	append gifAnim(filelist) " $file"
    } else {
	if { $gifAnim(frame_files_format) == "PPM" } {
	    # PPM format:
	    set file $head.ppm
	    append gifAnim(filelist) "$file\n"
	} else {
	    # JPEG format:
	    set file $head.jpg
	    append gifAnim(filelist) "$head.jpg\n"
	}
    }

    incr gifAnim(nframe)

    # now update the display (maybe $can was obscured)
    scripting::printToFile [file join $dir $file]
}


proc gifAnim:editParam {comlin} {
    global gifAnim system
    
    set comlin [subst {
# ------------------------------------------------------------------------
# Please edit the WHIRLGIF command-line (don't touch this comment)
# ------------------------------------------------------------------------

$comlin
}]

    if { $gifAnim(temp_files_dir) == "pwd" } {
	set dir $system(PWD)
    } else {
	set dir $system(SCRDIR)
    }	    

    set file [file join $dir whirlgif.param]
    WriteFile $file $comlin w
    xcEditFile $file foreground

    # drop the comments from file

    set content {}
    foreach line [split [ReadFile -nonewline $file] \n] {
	if { [string match *\#* $line] == 0 } {
	    if { $line != {} } {
		append content [format "%s\n" $line]
	    }
	}
    }
    return $content
}


    
#proc gifAnim:editParam {comlin} {
#    global gifAnim
#
#    set gifAnim(whirlgif_comlin) $comlin
#
#    set t [xcToplevel [WidgetName] "Edit Whirlgif command-line" "Edit"]
#    
#    set f1 [frame $t.1 -relief groove -bd 2]
#    set f2 [frame $t.2]
#    pack $f1 -side top -padx 5 -pady 5 -fill both -expand 1
#    pack $f2 -side top -padx 5 -pady 5 -fill x
#    
#    FillEntries $f1 {
#	"Edit Whirlgif command-line:"
#    } gifAnim(whirlgif_comlin) 30 80 top top
#    
#    set ok  [button $f2.ok  -text "OK"     -command [list gifAnim:_editParam ok  $t $comlin]]
#    set can [button $f2.can -text "Cancel" -command [list gifAnim:_editParam can $t $comlin]]
#    pack $can $ok -side left -padx 5 -pady 5 -ipadx 3 -ipady 3
#    
#    tkwait window $t
#    return $gifAnim(whirlgif_comlin)
#}
#
#proc gifAnim:_editParam {mode t comlin} {
#    global gifAnim
#    if { $mode == "can" } {
#	set gifAnim(whirlgif_comlin) $comlin
#    } 
#    CancelProc $t
#}